"use strict";

const codeSign_1 = require("./codeSign");
const bluebird_1 = require("bluebird");
const awaiter_1 = require("./awaiter");
const platformPackager_1 = require("./platformPackager");
const path = require("path");
const promisifed_fs_1 = require("./promisifed-fs");
const util_1 = require("./util");
const promisifed_fs_2 = require("./promisifed-fs");
const __awaiter = awaiter_1.tsAwaiter;
Array.isArray(__awaiter);
class WinPackager extends platformPackager_1.PlatformPackager {
    constructor(info, cleanupTasks) {
        super(info);
        if (this.options.cscLink != null && this.options.cscKeyPassword != null && process.platform !== "darwin") {
            this.certFilePromise = codeSign_1.downloadCertificate(this.options.cscLink).then(path => {
                cleanupTasks.push(() => promisifed_fs_2.deleteFile(path, true));
                return path;
            });
        } else {
            this.certFilePromise = bluebird_1.Promise.resolve(null);
        }
    }
    getBuildConfigurationKey() {
        return "win";
    }
    pack(platform, arch, outDir) {
        if (this.options.dist) {
            const installerOut = outDir + "-installer";
            util_1.log("Removing %s", installerOut);
            return bluebird_1.Promise.all([super.pack(platform, arch, outDir), promisifed_fs_2.deleteDirectory(installerOut)]);
        } else {
            return super.pack(platform, arch, outDir);
        }
    }
    packageInDistributableFormat(outDir, customConfiguration, arch) {
        return __awaiter(this, void 0, void 0, function* () {
            let iconUrl = this.metadata.build.iconUrl;
            if (!iconUrl) {
                if (customConfiguration != null) {
                    iconUrl = customConfiguration.iconUrl;
                }
                if (!iconUrl) {
                    if (this.info.repositoryInfo != null) {
                        const info = yield this.info.repositoryInfo.getInfo(this);
                        if (info != null) {
                            iconUrl = `https://raw.githubusercontent.com/${ info.user }/${ info.project }/master/build/icon.ico`;
                        }
                    }
                    if (!iconUrl) {
                        throw new Error("iconUrl is not specified, please see https://github.com/develar/electron-complete-builder#in-short");
                    }
                }
            }
            const certificateFile = yield this.certFilePromise;
            const version = this.metadata.version;
            const outputDirectory = outDir + "-installer";
            const options = Object.assign({
                name: this.metadata.name,
                appDirectory: outDir,
                outputDirectory: outputDirectory,
                productName: this.metadata.name,
                version: version,
                description: this.metadata.description,
                authors: this.metadata.author,
                iconUrl: iconUrl,
                setupIcon: path.join(this.projectDir, "build", "icon.ico"),
                certificateFile: certificateFile,
                certificatePassword: this.options.cscKeyPassword
            }, customConfiguration);
            try {
                yield new bluebird_1.Promise((resolve, reject) => {
                    require("electron-winstaller-temp-fork").build(options, error => error == null ? resolve(null) : reject(error));
                });
            } catch (e) {
                if (e.message.indexOf("Unable to set icon") < 0) {
                    throw e;
                } else {
                    let fileInfo;
                    try {
                        fileInfo = yield promisifed_fs_1.stat(options.setupIcon);
                    } catch (e) {
                        throw new Error("Please specify correct setupIcon, file " + options.setupIcon + " not found");
                    }
                    if (fileInfo.isDirectory()) {
                        throw new Error("Please specify correct setupIcon, " + options.setupIcon + " is a directory");
                    }
                }
            }
            const appName = this.metadata.name;
            const archSuffix = arch === "x64" ? "-x64" : "";
            return yield bluebird_1.Promise.all([promisifed_fs_1.renameFile(path.join(outputDirectory, appName + "Setup.exe"), path.join(outputDirectory, appName + "Setup-" + version + archSuffix + ".exe")).then(it => this.dispatchArtifactCreated(it)), promisifed_fs_1.renameFile(path.join(outputDirectory, appName + "-" + version + "-full.nupkg"), path.join(outputDirectory, appName + "-" + version + archSuffix + "-full.nupkg")).then(it => this.dispatchArtifactCreated(it))]);
        });
    }
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = WinPackager;
//# sourceMappingURL=winPackager.js.map
